/* CELLEDIT.PRG - Generic cell editing using TBrowse

SYNTAX

   CellEdit( <b> ) -> NIL

COMPILING

   FOR LIBRARY:  /n /w /m
   FOR TESTING:  /n /w /m /dTESTING

*/

#include "inkey.ch"
MEMVAR GetList

#ifdef TESTING
#define K_SPACE   Asc( " " )
PROCEDURE Test()
   LOCAL nKey,b:= TBrowseDB( 03,05,21,74 )
   FIELD name
   USE names
   b:addColumn( TBColumnNew( "Rec#", ;
               {|| Str(RecNo(),6) }   ) )
   b:addColumn( TBColumnNew( "Name",;
               {|x| IIf(x==NIL,name,name:=x) } ) )
   b:addColumn( TBColumnNew( "Company",;
               FieldBlock( "company" ) ) )
   @ 02,04 TO 22,75 DOUBLE
   WHILE .T.
      WHILE NextKey()==0
         IF b:stabilize()
            EXIT
         ENDIF
      END
      nKey:= InKey( 0 )
      DO CASE
      CASE nKey == K_ESC;     EXIT
      CASE nKey == K_UP;      b:up()
      CASE nKey == K_DOWN;    b:down()
      CASE nKey == K_LEFT;    b:left()
      CASE nKey == K_RIGHT;   b:right()
      CASE nKey == K_SPACE;   CellEdit( b )
      ENDCASE
   END
   CLS
   RETURN
#endif

// Edit current TBrowse cell
PROCEDURE CellEdit( b )
   LOCAL c,uEditVar,nWidth,xScr
   LOCAL nRow:= Row(), nCol:= Col()
   LOCAL nTr,nLc,nBr,nRc
   LOCAL xConfirm:= Set( _SET_CONFIRM )
   WHILE !b:stabilize(); END
   c:= b:getColumn( b:colPos )
   IF !TBColEditable( c )
      Tone( 100,3 )
   ELSE
      uEditVar:= Eval( c:block )
      nWidth:= Min( b:colWidth( b:colPos ),;
               DispLen( uEditVar ) )
      nTr:= nRow-1; nLc:= nCol-1
      nBr:= nRow+1; nRc:= nCol+nWidth
      xScr:= SaveScreen(nTr,nLc,nBr,nRc )
      @ nTr,nLc TO nBr,nRc
      @ nRow,nCol GET uEditVar;
                  PICTURE "@@S" + ;
      LTrim(Str(nWidth)) SET CONFIRM ON
      READ
      SET CONFIRM ( xConfirm )
      DispBegin()
      IF LastKey() <> K_ESC
         Eval( c:block,uEditVar )
         RestScreen( nTr,nLc,nBr,nRc,xScr )
         b:refreshCurrent()
         WHILE !b:stabilize(); END
      ELSE
      RestScreen( nTr,nLc,nBr,nRc,xScr )
      ENDIF
      DispEnd()
   ENDIF
   RETURN

// Return true if column is editable.
FUNCTION TBColEditable( c )
   LOCAL uEditVar:= Eval( c:block )    // Get orig.
   Eval( c:block, Modify( uEditVar ) ) // New value
   IF !( uEditVar == Eval( c:block ) ) // Changed?
      Eval( c:block, uEditVar )        // Restore
      RETURN .T.
   ENDIF
   RETURN .F.

// Return display length of a value
FUNCTION DispLen( uValue )
   LOCAL nLen, cType:= ValType( uValue )
   DO CASE
   CASE cType == "C"
      nLen:= Len( uValue )
   CASE cType == "D"
      nLen:= Len( Set( _SET_DATEFORMAT ) )
   CASE cType == "L"
      nLen:= 1
   CASE cType == "N"
      nLen:= Len( Str( uValue ) )
   CASE cType == "U"
      nLen:= 3    // NIL
   OTHERWISE
      nLen:= 0
   ENDCASE
   RETURN nLen

// Return a value with same data type as "uValue"
// but with a different actual value.
STATIC FUNCTION Modify( uValue )
   LOCAL cType:= ValType( uValue )
   DO CASE
   CASE cType == "C"
      RETURN Chr( Asc( Left( uValue,1 ) ) + 1 )
   CASE cType == "L"
      RETURN !uValue
   CASE cType == "D"
      RETURN uValue++
   CASE cType == "N"
      RETURN IIf( uValue == 0,1,0 )
   ENDCASE
   RETURN NIL

// EOF

